@@ -28,6 +28,8 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 
+#include <systemd/sd-login.h>
+
 #include "gdm-common.h"
 #include "gdm-manager.h"
 #include "gdm-display-factory.h"
@@ -40,16 +42,17 @@
 #include "gdm-local-display.h"
 #include "gdm-legacy-display.h"
 
-#define GDM_LOCAL_DISPLAY_FACTORY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_LOCAL_DISPLAY_FACTORY, GdmLocalDisplayFactoryPrivate))
-
 #define GDM_DBUS_PATH                       "/org/gnome/DisplayManager"
 #define GDM_LOCAL_DISPLAY_FACTORY_DBUS_PATH GDM_DBUS_PATH "/LocalDisplayFactory"
 #define GDM_MANAGER_DBUS_NAME               "org.gnome.DisplayManager.LocalDisplayFactory"
 
 #define MAX_DISPLAY_FAILURES 5
+#define WAIT_TO_FINISH_TIMEOUT 10 /* seconds */
 
-struct GdmLocalDisplayFactoryPrivate
+struct _GdmLocalDisplayFactory
 {
+        GdmDisplayFactory              parent;
+
         GdmDBusLocalDisplayFactory *skeleton;
         GDBusConnection *connection;
         GHashTable      *used_display_numbers;
@@ -59,6 +62,12 @@ struct GdmLocalDisplayFactoryPrivate
 
         guint            seat_new_id;
         guint            seat_removed_id;
+
+#if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
+        char            *tty_of_active_vt;
+        guint            active_vt_watch_id;
+        guint            wait_to_finish_timeout_id;
+#endif
 };
 
 enum {
@@ -80,6 +89,9 @@ static void     on_display_status_changed               (GdmDisplay
 
 static gboolean gdm_local_display_factory_sync_seats    (GdmLocalDisplayFactory *factory);
 static gpointer local_display_factory_object = NULL;
+static gboolean lookup_by_session_id (const char *id,
+                                      GdmDisplay *display,
+                                      gpointer    user_data);
 
 G_DEFINE_TYPE (GdmLocalDisplayFactory, gdm_local_display_factory, GDM_TYPE_DISPLAY_FACTORY)
 
@@ -131,7 +143,7 @@ take_next_display_number (GdmLocalDisplayFactory *factory)
         ret = 0;
         list = NULL;
 
-        g_hash_table_foreach (factory->priv->used_display_numbers, (GHFunc)listify_hash, &list);
+        g_hash_table_foreach (factory->used_display_numbers, (GHFunc)listify_hash, &list);
         if (list == NULL) {
                 goto out;
         }
@@ -163,7 +175,7 @@ take_next_display_number (GdmLocalDisplayFactory *factory)
 
         /* now reserve this number */
         g_debug ("GdmLocalDisplayFactory: Reserving X display: %u", ret);
-        g_hash_table_insert (factory->priv->used_display_numbers, GUINT_TO_POINTER (ret), NULL);
+        g_hash_table_insert (factory->used_display_numbers, GUINT_TO_POINTER (ret), NULL);
 
         return ret;
 }
@@ -185,6 +197,19 @@ store_display (GdmLocalDisplayFactory *factory,
         gdm_display_store_add (store, display);
 }
 
+static gboolean
+gdm_local_display_factory_use_wayland (void)
+{
+#ifdef ENABLE_WAYLAND_SUPPORT
+        gboolean wayland_enabled = FALSE;
+        if (gdm_settings_direct_get_boolean (GDM_KEY_WAYLAND_ENABLE, &wayland_enabled)) {
+                if (wayland_enabled && g_file_test ("/usr/bin/Xwayland", G_FILE_TEST_IS_EXECUTABLE) )
+                        return TRUE;
+        }
+#endif
+        return FALSE;
+}
+
 /*
   Example:
   dbus-send --system --dest=org.gnome.DisplayManager \
@@ -208,6 +233,8 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
 
 #ifdef ENABLE_USER_DISPLAY_SERVER
         display = gdm_local_display_new ();
+        if (gdm_local_display_factory_use_wayland ())
+                g_object_set (G_OBJECT (display), "session-type", "wayland", NULL);
 #else
         if (display == NULL) {
                 guint32 num;
@@ -243,29 +270,56 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
         return ret;
 }
 
+static void
+finish_display_on_seat_if_waiting (GdmDisplayStore *display_store,
+                                   GdmDisplay      *display,
+                                   const char      *seat_id)
+{
+        if (gdm_display_get_status (display) != GDM_DISPLAY_WAITING_TO_FINISH)
+                return;
+
+        g_debug ("GdmLocalDisplayFactory: finish background display\n");
+        gdm_display_stop_greeter_session (display);
+        gdm_display_unmanage (display);
+        gdm_display_finish (display);
+}
+
+static void
+finish_waiting_displays_on_seat (GdmLocalDisplayFactory *factory,
+                                 const char             *seat_id)
+{
+        GdmDisplayStore *store;
+
+        store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
+
+        gdm_display_store_foreach (store,
+                                   (GdmDisplayStoreFunc) finish_display_on_seat_if_waiting,
+                                   (gpointer)
+                                   seat_id);
+}
+
 static void
 on_display_status_changed (GdmDisplay             *display,
                            GParamSpec             *arg1,
                            GdmLocalDisplayFactory *factory)
 {
         int              status;
-        GdmDisplayStore *store;
         int              num;
         char            *seat_id = NULL;
         char            *session_type = NULL;
+        char            *session_class = NULL;
         gboolean         is_initial = TRUE;
         gboolean         is_local = TRUE;
 
         num = -1;
         gdm_display_get_x11_display_number (display, &num, NULL);
 
-        store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
-
         g_object_get (display,
                       "seat-id", &seat_id,
                       "is-initial", &is_initial,
                       "is-local", &is_local,
                       "session-type", &session_type,
+                      "session-class", &session_class,
                       NULL);
 
         status = gdm_display_get_status (display);
@@ -273,36 +327,36 @@ on_display_status_changed (GdmDisplay             *display,
         g_debug ("GdmLocalDisplayFactory: display status changed: %d", status);
         switch (status) {
         case GDM_DISPLAY_FINISHED:
-                /* remove the display number from factory->priv->used_display_numbers
+                /* remove the display number from factory->used_display_numbers
                    so that it may be reused */
                 if (num != -1) {
-                        g_hash_table_remove (factory->priv->used_display_numbers, GUINT_TO_POINTER (num));
+                        g_hash_table_remove (factory->used_display_numbers, GUINT_TO_POINTER (num));
                 }
-                gdm_display_store_remove (store, display);
+                gdm_display_factory_queue_purge_displays (GDM_DISPLAY_FACTORY (factory));
 
                 /* if this is a local display, do a full resync.  Only
                  * seats without displays will get created anyway.  This
                  * ensures we get a new login screen when the user logs out,
                  * if there isn't one.
                  */
-                if (is_local) {
+                if (is_local && g_strcmp0 (session_class, "greeter") != 0) {
                         /* reset num failures */
-                        factory->priv->num_failures = 0;
+                        factory->num_failures = 0;
 
                         gdm_local_display_factory_sync_seats (factory);
                 }
                 break;
         case GDM_DISPLAY_FAILED:
-                /* leave the display number in factory->priv->used_display_numbers
+                /* leave the display number in factory->used_display_numbers
                    so that it doesn't get reused */
-                gdm_display_store_remove (store, display);
+                gdm_display_factory_queue_purge_displays (GDM_DISPLAY_FACTORY (factory));
 
                 /* Create a new equivalent display if it was static */
                 if (is_local) {
 
-                        factory->priv->num_failures++;
+                        factory->num_failures++;
 
-                        if (factory->priv->num_failures > MAX_DISPLAY_FAILURES) {
+                        if (factory->num_failures > MAX_DISPLAY_FAILURES) {
                                 /* oh shit */
                                 g_warning ("GdmLocalDisplayFactory: maximum number of X display failures reached: check X server log for errors");
                         } else {
@@ -323,6 +377,8 @@ on_display_status_changed (GdmDisplay             *display,
                 break;
         case GDM_DISPLAY_MANAGED:
                 break;
+        case GDM_DISPLAY_WAITING_TO_FINISH:
+                break;
         default:
                 g_assert_not_reached ();
                 break;
@@ -330,6 +386,7 @@ on_display_status_changed (GdmDisplay             *display,
 
         g_free (seat_id);
         g_free (session_type);
+        g_free (session_class);
 }
 
 static gboolean
@@ -350,6 +407,21 @@ lookup_by_seat_id (const char *id,
         return res;
 }
 
+static gboolean
+lookup_prepared_display_by_seat_id (const char *id,
+                                    GdmDisplay *display,
+                                    gpointer    user_data)
+{
+        int status;
+
+        status = gdm_display_get_status (display);
+
+        if (status != GDM_DISPLAY_PREPARED)
+                return FALSE;
+
+        return lookup_by_seat_id (id, display, user_data);
+}
+
 static GdmDisplay *
 create_display (GdmLocalDisplayFactory *factory,
                 const char             *seat_id,
@@ -358,14 +430,41 @@ create_display (GdmLocalDisplayFactory *factory,
 {
         GdmDisplayStore *store;
         GdmDisplay      *display = NULL;
+        g_autofree char *login_session_id = NULL;
+
+        g_debug ("GdmLocalDisplayFactory: %s login display for seat %s requested",
+                 session_type? : "X11", seat_id);
+        store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
+
+        if (sd_seat_can_multi_session (seat_id))
+                display = gdm_display_store_find (store, lookup_prepared_display_by_seat_id, (gpointer) seat_id);
+        else
+                display = gdm_display_store_find (store, lookup_by_seat_id, (gpointer) seat_id);
 
         /* Ensure we don't create the same display more than once */
-        store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
-        display = gdm_display_store_find (store, lookup_by_seat_id, (gpointer) seat_id);
         if (display != NULL) {
+                g_debug ("GdmLocalDisplayFactory: display already created");
                 return NULL;
         }
 
+        /* If we already have a login window, switch to it */
+        if (gdm_get_login_window_session_id (seat_id, &login_session_id)) {
+                GdmDisplay *display;
+
+                display = gdm_display_store_find (store,
+                                                  lookup_by_session_id,
+                                                  (gpointer) login_session_id);
+                if (display != NULL &&
+                    (gdm_display_get_status (display) == GDM_DISPLAY_MANAGED ||
+                     gdm_display_get_status (display) == GDM_DISPLAY_WAITING_TO_FINISH)) {
+                        g_object_set (G_OBJECT (display), "status", GDM_DISPLAY_MANAGED, NULL);
+                        g_debug ("GdmLocalDisplayFactory: session %s found, activating.",
+                                 login_session_id);
+                        gdm_activate_session_by_id (factory->connection, seat_id, login_session_id);
+                        return NULL;
+                }
+        }
+
         g_debug ("GdmLocalDisplayFactory: Adding display on seat %s", seat_id);
 
 #ifdef ENABLE_USER_DISPLAY_SERVER
@@ -421,7 +520,8 @@ gdm_local_display_factory_sync_seats (GdmLocalDisplayFactory *factory)
         GVariantIter iter;
         const char *seat;
 
-        result = g_dbus_connection_call_sync (factory->priv->connection,
+        g_debug ("GdmLocalDisplayFactory: enumerating seats from logind");
+        result = g_dbus_connection_call_sync (factory->connection,
                                               "org.freedesktop.login1",
                                               "/org/freedesktop/login1",
                                               "org.freedesktop.login1.Manager",
@@ -447,14 +547,8 @@ gdm_local_display_factory_sync_seats (GdmLocalDisplayFactory *factory)
 
                 if (g_strcmp0 (seat, "seat0") == 0) {
                         is_initial = TRUE;
-#ifdef ENABLE_WAYLAND_SUPPORT
-                        gboolean wayland_enabled = FALSE;
-                        if (gdm_settings_direct_get_boolean (GDM_KEY_WAYLAND_ENABLE, &wayland_enabled)) {
-                                if (wayland_enabled && g_file_test ("/usr/bin/Xwayland", G_FILE_TEST_IS_EXECUTABLE) ) {
-                                        session_type = "wayland";
-                                }
-                        }
-#endif
+                        if (gdm_local_display_factory_use_wayland ())
+                                session_type = "wayland";
                 } else {
                         is_initial = FALSE;
                 }
@@ -497,10 +591,194 @@ on_seat_removed (GDBusConnection *connection,
         delete_display (GDM_LOCAL_DISPLAY_FACTORY (user_data), seat);
 }
 
+static gboolean
+lookup_by_session_id (const char *id,
+                      GdmDisplay *display,
+                      gpointer    user_data)
+{
+        const char *looking_for = user_data;
+        const char *current;
+
+        current = gdm_display_get_session_id (display);
+        return g_strcmp0 (current, looking_for) == 0;
+}
+
+#if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
+static gboolean
+wait_to_finish_timeout (GdmLocalDisplayFactory *factory)
+{
+        finish_waiting_displays_on_seat (factory, "seat0");
+        factory->wait_to_finish_timeout_id = 0;
+        return G_SOURCE_REMOVE;
+}
+
+static void
+maybe_stop_greeter_in_background (GdmLocalDisplayFactory *factory,
+                                  GdmDisplay             *display)
+{
+        g_autofree char *display_session_type = NULL;
+        gboolean doing_initial_setup = FALSE;
+
+        if (gdm_display_get_status (display) != GDM_DISPLAY_MANAGED) {
+                g_debug ("GdmLocalDisplayFactory: login window not in managed state, so ignoring");
+                return;
+        }
+
+        g_object_get (G_OBJECT (display),
+                      "session-type", &display_session_type,
+                      "doing-initial-setup", &doing_initial_setup,
+                      NULL);
+
+        /* we don't ever stop initial-setup implicitly */
+        if (doing_initial_setup) {
+                g_debug ("GdmLocalDisplayFactory: login window is performing initial-setup, so ignoring");
+                return;
+        }
+
+        /* we can only stop greeter for wayland sessions, since
+         * X server would jump back on exit */
+        if (g_strcmp0 (display_session_type, "wayland") != 0) {
+                g_debug ("GdmLocalDisplayFactory: login window is running on Xorg, so ignoring");
+                return;
+        }
+
+        g_debug ("GdmLocalDisplayFactory: killing login window once its unused");
+        g_object_set (G_OBJECT (display), "status", GDM_DISPLAY_WAITING_TO_FINISH, NULL);
+
+        /* We stop the greeter after a timeout to avoid flicker */
+        if (factory->wait_to_finish_timeout_id != 0)
+                g_source_remove (factory->wait_to_finish_timeout_id);
+
+        factory->wait_to_finish_timeout_id =
+                g_timeout_add_seconds (WAIT_TO_FINISH_TIMEOUT,
+                                       (GSourceFunc)wait_to_finish_timeout,
+                                       factory);
+}
+
+static gboolean
+on_vt_changed (GIOChannel    *source,
+               GIOCondition   condition,
+               GdmLocalDisplayFactory *factory)
+{
+        GIOStatus status;
+        static const char *tty_of_initial_vt = "tty" GDM_INITIAL_VT;
+        g_autofree char *tty_of_previous_vt = NULL;
+        g_autofree char *tty_of_active_vt = NULL;
+        g_autofree char *login_session_id = NULL;
+        g_autofree char *active_session_id = NULL;
+        const char *session_type = NULL;
+        int ret;
+
+        g_debug ("GdmLocalDisplayFactory: received VT change event");
+        g_io_channel_seek_position (source, 0, G_SEEK_SET, NULL);
+
+        if (condition & G_IO_PRI) {
+                g_autoptr (GError) error = NULL;
+                status = g_io_channel_read_line (source, &tty_of_active_vt, NULL, NULL, &error);
+
+                if (error != NULL) {
+                        g_warning ("could not read active VT from kernel: %s", error->message);
+                }
+                switch (status) {
+                        case G_IO_STATUS_ERROR:
+                            return G_SOURCE_REMOVE;
+                        case G_IO_STATUS_EOF:
+                            return G_SOURCE_REMOVE;
+                        case G_IO_STATUS_AGAIN:
+                            return G_SOURCE_CONTINUE;
+                        case G_IO_STATUS_NORMAL:
+                            break;
+                }
+        }
+
+        if ((condition & G_IO_ERR) || (condition & G_IO_HUP)) {
+                g_debug ("GdmLocalDisplayFactory: kernel hung up active vt watch");
+                return G_SOURCE_REMOVE;
+        }
+
+        if (tty_of_active_vt == NULL) {
+                g_debug ("GdmLocalDisplayFactory: unable to read active VT from kernel");
+                return G_SOURCE_CONTINUE;
+        }
+
+        g_strchomp (tty_of_active_vt);
+
+        /* don't do anything if we're on the same VT we were before */
+        if (g_strcmp0 (tty_of_active_vt, factory->tty_of_active_vt) == 0) {
+                g_debug ("GdmLocalDisplayFactory: VT changed to the same VT, ignoring");
+                return G_SOURCE_CONTINUE;
+        }
+
+        tty_of_previous_vt = g_steal_pointer (&factory->tty_of_active_vt);
+        factory->tty_of_active_vt = g_steal_pointer (&tty_of_active_vt);
+
+        /* don't do anything at start up */
+        if (tty_of_previous_vt == NULL) {
+                g_debug ("GdmLocalDisplayFactory: VT is %s at startup",
+                         factory->tty_of_active_vt);
+                return G_SOURCE_CONTINUE;
+        }
+
+        g_debug ("GdmLocalDisplayFactory: VT changed from %s to %s",
+                 tty_of_previous_vt, factory->tty_of_active_vt);
+
+        /* if the old VT was running a wayland login screen kill it
+         */
+        if (gdm_get_login_window_session_id ("seat0", &login_session_id)) {
+                unsigned int vt;
+
+                ret = sd_session_get_vt (login_session_id, &vt);
+                if (ret == 0 && vt != 0) {
+                        g_autofree char *tty_of_login_window_vt = NULL;
+
+                        tty_of_login_window_vt = g_strdup_printf ("tty%u", vt);
+
+                        g_debug ("GdmLocalDisplayFactory: tty of login window is %s", tty_of_login_window_vt);
+                        if (g_strcmp0 (tty_of_login_window_vt, tty_of_previous_vt) == 0) {
+                                GdmDisplayStore *store;
+                                GdmDisplay *display;
+
+                                g_debug ("GdmLocalDisplayFactory: VT switched from login window");
+
+                                store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
+                                display = gdm_display_store_find (store,
+                                                                  lookup_by_session_id,
+                                                                  (gpointer) login_session_id);
+
+                                if (display != NULL)
+                                        maybe_stop_greeter_in_background (factory, display);
+                        } else {
+                                g_debug ("GdmLocalDisplayFactory: VT not switched from login window");
+                        }
+                }
+        }
+
+        /* if user jumped back to initial vt and it's empty put a login screen
+         * on it (unless a login screen is already running elsewhere, then
+         * jump to that login screen)
+         */
+        if (strcmp (factory->tty_of_active_vt, tty_of_initial_vt) != 0) {
+                g_debug ("GdmLocalDisplayFactory: active VT is not initial VT, so ignoring");
+                return G_SOURCE_CONTINUE;
+        }
+
+        if (gdm_local_display_factory_use_wayland ())
+                session_type = "wayland";
+
+        g_debug ("GdmLocalDisplayFactory: creating new display on seat0 because of VT change");
+
+        create_display (factory, "seat0", session_type, TRUE);
+
+        return G_SOURCE_CONTINUE;
+}
+#endif
+
 static void
 gdm_local_display_factory_start_monitor (GdmLocalDisplayFactory *factory)
 {
-        factory->priv->seat_new_id = g_dbus_connection_signal_subscribe (factory->priv->connection,
+        g_autoptr (GIOChannel) io_channel = NULL;
+
+        factory->seat_new_id = g_dbus_connection_signal_subscribe (factory->connection,
                                                                          "org.freedesktop.login1",
                                                                          "org.freedesktop.login1.Manager",
                                                                          "SeatNew",
@@ -510,7 +788,7 @@ gdm_local_display_factory_start_monitor (GdmLocalDisplayFactory *factory)
                                                                          on_seat_new,
                                                                          g_object_ref (factory),
                                                                          g_object_unref);
-        factory->priv->seat_removed_id = g_dbus_connection_signal_subscribe (factory->priv->connection,
+        factory->seat_removed_id = g_dbus_connection_signal_subscribe (factory->connection,
                                                                              "org.freedesktop.login1",
                                                                              "org.freedesktop.login1.Manager",
                                                                              "SeatRemoved",
@@ -520,21 +798,46 @@ gdm_local_display_factory_start_monitor (GdmLocalDisplayFactory *factory)
                                                                              on_seat_removed,
                                                                              g_object_ref (factory),
                                                                              g_object_unref);
+
+#if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
+        io_channel = g_io_channel_new_file ("/sys/class/tty/tty0/active", "r", NULL);
+
+        if (io_channel != NULL) {
+                factory->active_vt_watch_id =
+                        g_io_add_watch (io_channel,
+                                        G_IO_PRI,
+                                        (GIOFunc)
+                                        on_vt_changed,
+                                        factory);
+        }
+#endif
 }
 
 static void
 gdm_local_display_factory_stop_monitor (GdmLocalDisplayFactory *factory)
 {
-        if (factory->priv->seat_new_id) {
-                g_dbus_connection_signal_unsubscribe (factory->priv->connection,
-                                                      factory->priv->seat_new_id);
-                factory->priv->seat_new_id = 0;
+        if (factory->seat_new_id) {
+                g_dbus_connection_signal_unsubscribe (factory->connection,
+                                                      factory->seat_new_id);
+                factory->seat_new_id = 0;
         }
-        if (factory->priv->seat_removed_id) {
-                g_dbus_connection_signal_unsubscribe (factory->priv->connection,
-                                                      factory->priv->seat_removed_id);
-                factory->priv->seat_removed_id = 0;
+        if (factory->seat_removed_id) {
+                g_dbus_connection_signal_unsubscribe (factory->connection,
+                                                      factory->seat_removed_id);
+                factory->seat_removed_id = 0;
         }
+#if defined(ENABLE_WAYLAND_SUPPORT) && defined(ENABLE_USER_DISPLAY_SERVER)
+        if (factory->wait_to_finish_timeout_id != 0) {
+                g_source_remove (factory->wait_to_finish_timeout_id);
+                factory->wait_to_finish_timeout_id = 0;
+        }
+        if (factory->active_vt_watch_id) {
+                g_source_remove (factory->active_vt_watch_id);
+                factory->active_vt_watch_id = 0;
+        }
+
+        g_clear_pointer (&factory->tty_of_active_vt, g_free);
+#endif
 }
 
 static void
@@ -558,18 +861,11 @@ on_display_added (GdmDisplayStore        *display_store,
 
 static void
 on_display_removed (GdmDisplayStore        *display_store,
-                    const char             *id,
+                    GdmDisplay             *display,
                     GdmLocalDisplayFactory *factory)
 {
-        GdmDisplay *display;
-
-        display = gdm_display_store_lookup (display_store, id);
-
-        if (display != NULL) {
-                g_signal_handlers_disconnect_by_func (display, G_CALLBACK (on_display_status_changed), factory);
-                g_object_weak_unref (G_OBJECT (display), (GWeakNotify)on_display_disposed, factory);
-
-        }
+        g_signal_handlers_disconnect_by_func (display, G_CALLBACK (on_display_status_changed), factory);
+        g_object_weak_unref (G_OBJECT (display), (GWeakNotify)on_display_disposed, factory);
 }
 
 static gboolean
@@ -674,22 +970,22 @@ register_factory (GdmLocalDisplayFactory *factory)
         GError *error = NULL;
 
         error = NULL;
-        factory->priv->connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
-        if (factory->priv->connection == NULL) {
+        factory->connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
+        if (factory->connection == NULL) {
                 g_critical ("error getting system bus: %s", error->message);
                 g_error_free (error);
                 exit (EXIT_FAILURE);
         }
 
-        factory->priv->skeleton = GDM_DBUS_LOCAL_DISPLAY_FACTORY (gdm_dbus_local_display_factory_skeleton_new ());
+        factory->skeleton = GDM_DBUS_LOCAL_DISPLAY_FACTORY (gdm_dbus_local_display_factory_skeleton_new ());
 
-        g_signal_connect (factory->priv->skeleton,
+        g_signal_connect (factory->skeleton,
                           "handle-create-transient-display",
                           G_CALLBACK (handle_create_transient_display),
                           factory);
 
-        if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (factory->priv->skeleton),
-                                               factory->priv->connection,
+        if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (factory->skeleton),
+                                               factory->connection,
                                                GDM_LOCAL_DISPLAY_FACTORY_DBUS_PATH,
                                                &error)) {
                 g_critical ("error exporting LocalDisplayFactory object: %s", error->message);
@@ -733,16 +1029,12 @@ gdm_local_display_factory_class_init (GdmLocalDisplayFactoryClass *klass)
 
         factory_class->start = gdm_local_display_factory_start;
         factory_class->stop = gdm_local_display_factory_stop;
-
-        g_type_class_add_private (klass, sizeof (GdmLocalDisplayFactoryPrivate));
 }
 
 static void
 gdm_local_display_factory_init (GdmLocalDisplayFactory *factory)
 {
-        factory->priv = GDM_LOCAL_DISPLAY_FACTORY_GET_PRIVATE (factory);
-
-        factory->priv->used_display_numbers = g_hash_table_new (NULL, NULL);
+        factory->used_display_numbers = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -755,12 +1047,12 @@ gdm_local_display_factory_finalize (GObject *object)
 
         factory = GDM_LOCAL_DISPLAY_FACTORY (object);
 
-        g_return_if_fail (factory->priv != NULL);
+        g_return_if_fail (factory != NULL);
 
-        g_clear_object (&factory->priv->connection);
-        g_clear_object (&factory->priv->skeleton);
+        g_clear_object (&factory->connection);
+        g_clear_object (&factory->skeleton);
 
-        g_hash_table_destroy (factory->priv->used_display_numbers);
+        g_hash_table_destroy (factory->used_display_numbers);
 
         gdm_local_display_factory_stop_monitor (factory);
 
